//////////////////////////////////////////////////////////////////////////////
// DOM Storage用ライブラリ
//							(C)2010 nabe@abk
//////////////////////////////////////////////////////////////////////////////
//[TAB=4]
/*----------------------------------------------------------------------------
Copyright 2010 The nabe@abk. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
//////////////////////////////////////////////////////////////////////////////
// ■ switchクラス（開く、閉じるを切り変える）
//////////////////////////////////////////////////////////////////////////////
function HSwitch(path, arg) {
	this.ls = load_PrefixStorage(path, arg);
	this.sw  = new Array;
	this.len = new Array;
	this.obj = new Array;
}
HSwitch.prototype.regist = function(id, class0, class1) {
	if (class0==undefined) class0='off';
	if (class1==undefined) class1='on';
	this.sw[id]  = [class0, class1];
	this.len[id] = 2;
	this.obj[id]      = document.getElementById(id);
	this.obj[id+"sw"] = document.getElementById(id+":sw");
	// default setting
	this.set(id);
}
HSwitch.prototype.change = function(id) {
	var next = this.ls.getInt(id) +1;
	if (next >= this.len[id]) next=0;
	this.ls.set(id, next);
	this.set(id);
}
HSwitch.prototype.on = function(id) {
	this.ls.set(id, 1);
	this.set(id);
}
HSwitch.prototype.off = function(id) {
	this.ls.set(id, 0);
	this.set(id);
}
HSwitch.prototype.set = function(id) {
	var cl = this.sw[id][ this.ls.getInt(id) ];
	this.obj[id].className = cl;
	if (this.obj[id+"sw"])
		this.obj[id+"sw"].className = "switch-"+cl;
}

//////////////////////////////////////////////////////////////////////////////
// ■ Prefix付 Storage の取得
//////////////////////////////////////////////////////////////////////////////
//
// ・pathを適切に設定することで、同一ドメイン内で住み分けることができる。
// ・ただし紳士協定に過ぎないので過剰な期待は禁物
// is_session  0(default):LocalStorage 1:sessionStorage 2:ワンタイムなobj
//
//（利用可能メソッド） set(), get(), remove(), clear()
//
function load_PrefixStorage(path, is_session) {
	/*@cc_on
	if (@_jscript_version <= 5.7) {	// IE6, IE7
		if (!is_session || is_session < 2)
			return new userDataBehavior(path);
	}
	@*/
	var ls = new PrefixStorage(path, is_session);
	return ls;	// fail
}
function PrefixStorage(path, is_session) {
	// ローカルストレージのロード
	this.ls = Load_DOMStorage( is_session );

	// プレフィックス
	this.prefix = String(path) + '::';
}

//-------------------------------------------------------------------
// メンバ関数
//-------------------------------------------------------------------
PrefixStorage.prototype.set = function (key,val) {
	this.ls[this.prefix + key] = val;
};
PrefixStorage.prototype.get = function (key) {
	return this.ls[this.prefix + key];
};
PrefixStorage.prototype.getInt = function (key) {
	var v = this.ls[this.prefix + key];
	if (v==undefined) return 0;
	return parseInt(v);
};
PrefixStorage.prototype.remove = function(key) {
	this.ls.removeItem(this.prefix + key);
};
PrefixStorage.prototype.allclear = function() {
	this.ls.clear();
};
PrefixStorage.prototype.clear = function(key) {
	var ls = this.ls;
	var pf = this.prefix;
	var len = pf.length;

	if (ls.length != undefined) {
		var ary = new Array();
		for(var i=0; i<ls.length; i++) {
			var k = ls.key(i);
			if (k.substr(0,len) === pf)	ary.push(k);
		}
		// forでkey取り出し中には削除しない
		//（理由はDOM Storage仕様書参照のこと）
		for(var i in ary) {
			delete ls[ ary[i] ];
		}
	} else {
		// DOMStorageDummy
		for(var k in ls) {
			if (k.substr(0,len) === pf)
				delete ls[k];
		}
	}
};

//////////////////////////////////////////////////////////////////////////////
// ■ Storageの取得
//////////////////////////////////////////////////////////////////////////////
//(参考資料) http://www.html5.jp/trans/w3c_webstorage.html
function Load_DOMStorage(is_session) {
	var storage;
	try {
		// LocalStorage
		if (typeof(localStorage) != "object" && typeof(globalStorage) == "object") {
			localStorage = globalStorage[location.host];
		}
		// ストレージの取得
		storage = is_session ? sessionStorage : localStorage;
	} catch(e) {
		storage = undefined;
	}

	// 未定義のとき DOM Storage もどきをロード
	if (!storage || is_session==2) {
		storage = new DOMStorageDummy();
	}
	return storage;
}

//////////////////////////////////////////////////////////////////////////////
// ■ sessionStorage 程度の DOM Storage もどき
//////////////////////////////////////////////////////////////////////////////
// length, storageイベントは非対応
//
function DOMStorageDummy() {
	// メンバ関数
	DOMStorageDummy.prototype.setItem = function(key, val) { this[key] = val; };
	DOMStorageDummy.prototype.getItem = function(key) { return this[key]; };
	DOMStorageDummy.prototype.removeItem = function(key) { delete this[key]; };
	DOMStorageDummy.prototype.clear = function() {
		for(var k in this) {
			if(typeof(this[k]) == 'function') continue;
			delete this[k];
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// ■ IE6/7の userData behavior によるローカルストレージ
//////////////////////////////////////////////////////////////////////////////
// ・同一ディレクトリのみ参照可能。
// ・headロード後でないと使用できない。
/*@cc_on
if (@_jscript_version <= 5.7) {	// IE6, IE7
//----------------------------------------------------------------------------
var userDataBehavior_obj = new Object;	// cacheされたストレージ
var userDataBehavior_div;

function userDataBehavior(path) {
	this.sn = 'bstorage';

	if (! userDataBehavior_div) {
		var div = document.createElement('div');
		div.style.behavior = 'url(#default#userData)';
		div.style.display  = 'block';
		// Append
		document.getElementsByTagName('head')[0].appendChild(div);
		userDataBehavior_div = div;

		// Behaviorストレージからロード
		var obj = userDataBehavior_obj;
		div.load(this.sn);
		var ary = (div.getAttribute( this.sn ) || '').split(' ');
		while(ary.length > 1) {
			var k = ary.shift();
			var v = ary.shift();
			obj[k] = v;
		}
	}

	// メンバ変数
	this.ls  = userDataBehavior_obj;
	this.div = userDataBehavior_div;
	this.prefix = String(path) + '::';
}
// 
userDataBehavior.prototype.set = function (key,val) {
	this.ls[this.prefix + key] = val;
	this.update();
};
userDataBehavior.prototype.get = function (key) {
	return this.ls[this.prefix + key];
};
userDataBehavior.prototype.getInt = function (key) {
	var v = this.ls[this.prefix + key];
	if (v==undefined) return 0;
	return Number(v);
};
userDataBehavior.prototype.remove = function(key) {
	delete this.ls[this.prefix + key];
	this.update();
};
userDataBehavior.prototype.clear = function(key) {
	var len = this.prefix.length;
	// DOMStorageDummy
	for(var k in this.ls) {
		if (k.substr(0,len) === this.prefix)
			delete this.ls[k];
	}
	this.update();
};
userDataBehavior.prototype.allclear = function() {
	this.ls = new Object;
	this.update();
};
// Behaviorストレージに保存する
userDataBehavior.prototype.update = function ()  {
	var ls=this.ls;
	var div=this.div;
	var str='';
	for(var k in ls) {
		str += encodeURI(k)+' '+ls[k]+' ';
	}
	str.chop;
	if (str == '') {
		div.removeAttribute(this.sn);
	} else {
		div.setAttribute(this.sn, str);
	}
	div.save(this.sn);
};
} @*/

